//#include "tools.h"
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <errno.h>

#define BAUDRATE B19200
#define MODEMDEVICE "/dev/ttyS0"
#define _POSIX_SOURCE 1                 //POSIX compliant source

#define DWORD unsigned long
#define FALSE 0
#define TRUE 1

#define SLEEP(x) usleep(x*10)
#define READ_TIMEOUT_UNIT  2            // units:READ_TIMEOUT_UNIT(ms)
#define READ_TIMEOUT_CNT   5 

char message[256];
volatile int STOP = FALSE;
void signal_handler_IO (int status);    //definition of signal handler
int wait_flag=TRUE;                     //TRUE while no signal received
char devicename[80];
long Baud_Rate = 19200;                 // default Baud Rate (110 through 38400)
long BAUD;                              // derived baud rate from command line
long DATABITS;
long STOPBITS;
long PARITYON;
long PARITY;
int Data_Bits = 8;              // Number of data bits
int Stop_Bits = 1;              // Number of stop bits
int Parity = 0;                 // Parity as follows: 
                // 00 = NONE, 01 = Odd, 02 = Even, 03 = Mark, 04 = Space
int Format = 4;
int status;
int fd;

struct termios oldtio, newtio;

int initSerialPort(unsigned short usCOM_Num)
{
    int c, i, error;
    char In1, Key;
//  struct termios oldtio, newtio;
    struct sigaction saio;               //definition of signal action
    char buf[255];                       //buffer for where data is put
    sprintf(devicename, "/dev/ttyS%01d", usCOM_Num - 1);
    BAUD =  B9600;
    DATABITS = CS8;
    PARITYON = 0;
    STOPBITS = 0;
        fd = open(devicename, O_RDWR | O_NOCTTY | O_NONBLOCK);
    if (fd < 0)
        {
                perror(devicename);
                sprintf(message,"Open %s fail!!\r\n",devicename);
                fd = 0;
                return 0;
        }

    //install the serial handler before making the device asynchronous
    tcgetattr(fd, &oldtio);     // save current port settings 
//debug 
//  cfsetispeed(&newtio, B9600);
//  cfsetospeed(&newtio, B9600); 
    newtio = oldtio; 
    newtio.c_cflag = (BAUD | CS8 | CREAD | CLOCAL | HUPCL);// | DATABITS | STOPBITS);
    newtio.c_iflag = IGNPAR;
    newtio.c_oflag = 0;
    newtio.c_lflag = 0;        //ICANON;
    newtio.c_cc[VMIN]=1;
    newtio.c_cc[VTIME]=0;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &newtio);              
    int n;
    for(n=0; n < 3; n++)
    {
        tcflush(fd, TCIOFLUSH);
        usleep(100000);
    }
    return fd;
} //end of initSerialPort

int writeSerial(unsigned char *buf, int bufsize)
{
    int  bytes_sent = 0;
    int  attempts = 0;
    int  max_send_attempts = 100;
    while( 1 )
    {
        int actualy_bytes_sent = write(fd, buf+bytes_sent, bufsize-bytes_sent);
        if(actualy_bytes_sent == -1 )
        {
            if( errno == EAGAIN )
            {
                if( attempts < max_send_attempts)
                {
                    usleep(10000); // 100 ms
                    attempts++;
                }
                else
                {
                    return bytes_sent;
                }
                continue;
            }
            else
            {
                return -1;
            }
        } else {
            attempts = 0;
        }

        bytes_sent += actualy_bytes_sent;
        if( bytes_sent == bufsize )
            break;
        if( bytes_sent > bufsize )
            return -1;
        usleep(1);
    }
    return bytes_sent;
}

int readSerial(unsigned char *buf, int bufsize)
{
    int bytes_available;
    int attempt = 0;
    int max_read_attempt = 1000;
    while( 1 )
    {
        ioctl(fd, FIONREAD, &bytes_available );
        if(bytes_available < bufsize)
        {
            if( attempt < max_read_attempt )
            {
                usleep(1000);
                attempt++;
            }
            else
            {
                break;
            }
        }
        else
        {
            bytes_available = bufsize;
            break;
        }       
    }

    if( bytes_available > 0 )
    {
        bufsize=read(fd, buf, bytes_available);
        return bufsize;
    }
    return bytes_available;
}

void CloseSerialPort()
{
    close(fd);        //close the com port
}

void signal_handler_IO (int status)
{
    wait_flag = FALSE;
}
